@@ -125,10 +125,10 @@ namespace ROCWMMA_TYPE_TRAITS_IMPL_NAMESPACE
 // 1. Re-enable static assert
 // 2. Give ROCWMMA_FP8_FNUZ value of 1
 
-//static_assert((bool)ROCWMMA_ARCH_GFX94X || (bool)ROCWMMA_ARCH_HOST,
-//              "fp8_fnuz types only supported on gfx94X archs");
+//static_assert((bool)ROCWMMA_ARCH_GFX942 || (bool)ROCWMMA_ARCH_HOST,
+//              "fp8_fnuz types only supported on gfx942 arch");
 
-#define ROCWMMA_FP8_FNUZ (ROCWMMA_ARCH_GFX94X || ROCWMMA_ARCH_HOST)
+#define ROCWMMA_FP8_FNUZ (ROCWMMA_ARCH_GFX942 || ROCWMMA_ARCH_HOST)
 #define ROCWMMA_FP8_FNUZ_VISIBILITY ROCWMMA_HOST_DEVICE
 #else
 #define ROCWMMA_FP8_FNUZ 0
@@ -144,7 +144,7 @@ namespace ROCWMMA_TYPE_TRAITS_IMPL_NAMESPACE
 //static_assert((bool)ROCWMMA_ARCH_GFX12 || (bool)ROCWMMA_ARCH_HOST,
 //              "fp8_fnuz types only supported on gfx12 archs");
 
-#define ROCWMMA_FP8 (ROCWMMA_ARCH_GFX12 || ROCWMMA_ARCH_HOST)
+#define ROCWMMA_FP8 (ROCWMMA_ARCH_GFX950 || ROCWMMA_ARCH_GFX12 || ROCWMMA_ARCH_HOST)
 #define ROCWMMA_FP8_VISIBILITY ROCWMMA_HOST_DEVICE
 #else
 
@@ -153,20 +153,20 @@ namespace ROCWMMA_TYPE_TRAITS_IMPL_NAMESPACE
 #endif // defined(HIP_FP8_TYPE_OCP) && HIP_FP8_TYPE_OCP
 
 #if defined(__HIPCC_RTC__)
-    #if ROCWMMA_FP8_FNUZ
-	#define  ENABLE_FNUZ_HIPRTC 1
-    #else
-	#define ENABLE_FNUZ_HIPRTC 0
-    #endif
-
-    #if ROCWMMA_FP8_OCP
-	#define ENABLE_OCP_HIPRTC 1
-    #else
-	#define ENABLE_OCP_HIPRTC 0
-    #endif
+#if ROCWMMA_FP8_FNUZ
+#define ROCWMMA_ENABLE_FNUZ_HIPRTC 1
+#else
+#define ROCWMMA_ENABLE_FNUZ_HIPRTC 0
 #endif
 
-#if !defined(ENABLE_OCP_HIPRTC) || ENABLE_OCP_HIPRTC
+#if ROCWMMA_FP8_OCP
+#define ROCWMMA_ENABLE_OCP_HIPRTC 1
+#else
+#define ROCWMMA_ENABLE_OCP_HIPRTC 0
+#endif
+#endif
+
+#if !defined(ROCWMMA_ENABLE_OCP_HIPRTC) || ROCWMMA_ENABLE_OCP_HIPRTC
 ROCWMMA_FP8_VISIBILITY constexpr inline hip_fp8_e4m3
     make_hip_fp8_e4m3_from_bits(__hip_fp8_storage_t bits)
 {
@@ -646,14 +646,14 @@ namespace ROCWMMA_NUMERIC_LIMITS_IMPL_NAMESPACE
         }
     };
 }
-#endif // ENABLE_OCP_HIPRTC
+#endif // ROCWMMA_ENABLE_OCP_HIPRTC
 //////////////////////////////////////////
 ///  FNUZ f8 / bf8 operator overloads  ///
 //////////////////////////////////////////
 
 using rocwmma::uint8_t;
 
-#if !defined(ENABLE_FNUZ_HIPRTC) || ENABLE_FNUZ_HIPRTC
+#if !defined(ROCWMMA_ENABLE_FNUZ_HIPRTC) || ROCWMMA_ENABLE_FNUZ_HIPRTC
 ROCWMMA_FP8_FNUZ_VISIBILITY constexpr inline auto
     make_hip_fp8_e4m3_fnuz_from_bits(__hip_fp8_storage_t bits)
 {
@@ -837,7 +837,7 @@ ROCWMMA_FP8_FNUZ_VISIBILITY inline float operator*(hip_fp8_e4m3_fnuz a, float b)
     return (float(a) * b);
 }
 
-ROCWMMA_FP8_FNUZ_VISIBILITY inline float operator*(int32_t a, hip_fp8_e4m3_fnuz b)
+ROCWMMA_FP8_FNUZ_VISIBILITY inline float operator*(signed int a, hip_fp8_e4m3_fnuz b)
 {
     return ((float)a * float(b));
 }
@@ -862,7 +862,7 @@ ROCWMMA_FP8_FNUZ_VISIBILITY inline float operator*(hip_fp8_e5m2_fnuz a, float b)
     return (float(a) * b);
 }
 
-ROCWMMA_FP8_FNUZ_VISIBILITY inline float operator*(int32_t a, hip_fp8_e5m2_fnuz b)
+ROCWMMA_FP8_FNUZ_VISIBILITY inline float operator*(signed int a, hip_fp8_e5m2_fnuz b)
 {
     return ((float)a * float(b));
 }
@@ -899,7 +899,7 @@ ROCWMMA_FP8_FNUZ_VISIBILITY inline float operator/(hip_fp8_e4m3_fnuz a, float b)
     return (float(a) / b);
 }
 
-ROCWMMA_FP8_FNUZ_VISIBILITY inline float operator/(int32_t a, hip_fp8_e4m3_fnuz b)
+ROCWMMA_FP8_FNUZ_VISIBILITY inline float operator/(signed int a, hip_fp8_e4m3_fnuz b)
 {
     return ((float)a / float(b));
 }
@@ -924,7 +924,7 @@ ROCWMMA_FP8_FNUZ_VISIBILITY inline float operator/(hip_fp8_e5m2_fnuz a, float b)
     return (float(a) / b);
 }
 
-ROCWMMA_FP8_FNUZ_VISIBILITY inline float operator/(int32_t a, hip_fp8_e5m2_fnuz b)
+ROCWMMA_FP8_FNUZ_VISIBILITY inline float operator/(signed int a, hip_fp8_e5m2_fnuz b)
 {
     return ((float)a / float(b));
 }
@@ -1109,7 +1109,6 @@ namespace ROCWMMA_NUMERIC_LIMITS_IMPL_NAMESPACE
     //@endcond
 
 } // namespace ROCWMMA_NUMERIC_LIMITS_IMPL_NAMESPACE
-#endif // ENABLE_FNUZ_HIPRTC
+#endif // ROCWMMA_ENABLE_FNUZ_HIPRTC
 
 #endif // ROCWMMA_FLOAT8_HPP
-
