@@ -28,8 +28,11 @@
 
 #include "api_fwd.hpp"
 #include "constants.hpp"
+#include "io_scheduler.hpp"
 #include "layout/layout.hpp"
+#include "layout/matrix_coop_layout.hpp"
 #include "types.hpp"
+#include "utility/algorithm.hpp"
 
 namespace rocwmma
 {
@@ -67,9 +70,11 @@ namespace rocwmma
             // Basic non-interleaved layouts are classified into *OrthoVW (SOA) and *InlineVW (AOS) formats.
             // For any BlockDim/BlockK geometry, we ensure that these layouts come up with the same MaxVW,
             // so that the AOS <-> SOA transforms are possible and valid. The followings tests assure this.
-            static constexpr bool BlockKTest = (Constants::AMDGCN_WAVE_SIZE * TestWidth / min(BlockDim, Constants::AMDGCN_WAVE_SIZE)) <= BlockK;
-            static constexpr bool OrthoTest = TestWidth <= BlockK;
-            static constexpr bool InlineTest = TestWidth <= BlockDim;
+            static constexpr bool BlockKTest = (Constants::AMDGCN_WAVE_SIZE * TestWidth
+                                                / min(BlockDim, Constants::AMDGCN_WAVE_SIZE))
+                                               <= BlockK;
+            static constexpr bool OrthoTest         = TestWidth <= BlockK;
+            static constexpr bool InlineTest        = TestWidth <= BlockDim;
             static constexpr bool LayoutFitnessTest = (BlockKTest && OrthoTest && InlineTest);
 
             // Decide on final MaxVW
@@ -133,99 +138,75 @@ namespace rocwmma
     * @tparam BlockK Block K-dimension
     * @tparam DataT data type
     * @tparam DataLayoutT in-memory layout as col_major or row_major
-    * @tparam WaveCount number of cooperative waves
+    * @tparam Scheduler wave-wise scheduler
     */
     template <typename MatrixT,
               uint32_t BlockDim,
               uint32_t BlockK,
               typename DataT,
               typename DataLayoutT,
-              uint32_t WaveCount>
+              typename Scheduler>
     struct IOLayout;
 
     template <uint32_t BlockDim,
               uint32_t BlockK,
               typename DataT,
               typename DataLayoutT,
-              uint32_t WaveCount>
-    struct IOLayout<matrix_a, BlockDim, BlockK, DataT, DataLayoutT, WaveCount>
+              typename Scheduler>
+    struct IOLayout<matrix_a, BlockDim, BlockK, DataT, DataLayoutT, Scheduler>
     {
-        // Vector size properties
-        constexpr static uint32_t MaxVW = detail::
-            MaxVWSelector<matrix_a, BlockDim, BlockK, DataT, DataLayoutT, WaveCount>::Result;
+        using SchedulerTraits = scheduler_traits<Scheduler>;
 
+        // For non-interleaved layout, MmaDim is given by BlockDim
+        constexpr static uint32_t MmaDim = BlockDim;
+
+        // Large dim affects our layout choice
+        constexpr static bool LargeBlockDim = BlockDim > 32u;
+
+        // Determine optimal vector width properties
+        constexpr static uint32_t MaxVW = detail::MaxVWSelector<matrix_a,
+                                                                BlockDim,
+                                                                BlockK,
+                                                                DataT,
+                                                                DataLayoutT,
+                                                                SchedulerTraits::WaveCount>::Result;
+
+        // Note: ColOrthoVW in col_major is only valid for VW = 1
         constexpr static uint32_t VW
-            = is_same_v<DataLayoutT, row_major> || BlockDim > 32u ? MaxVW : 1u;
+            = (is_same_v<DataLayoutT, row_major> || LargeBlockDim) ? MaxVW : 1u;
 
-        // DataLayout
+        // DataLayout selection
         using DataLayout = DataLayout::template Array1d<DataLayoutT>;
 
-        // Matrix Layouts
-        // Small dim mma friendly
-        using SmallDimMatrixLayout
-            = MatrixLayout::ColOrthoVW<BlockDim, BlockK, DataT, VW, MaxVW>;
+        // MatrixLayout selection depends on some factors:
+        // - BlockDim
+        // - Cooperative wave mode
 
-        // Large dim not mma friendly
+        // Small dim: ensure our layout mma friendly
+        using SmallDimMatrixLayout = MatrixLayout::ColOrthoVW<BlockDim, BlockK, DataT, VW, MaxVW>;
+
+        // Large dim: we can use higher BW layouts, not mma friendly
         using LargeDimMatrixLayout
             = conditional_t<is_same_v<DataLayoutT, col_major>,
                             MatrixLayout::ColInlineVW<BlockDim, BlockK, DataT, VW, MaxVW>,
                             MatrixLayout::ColOrthoVW<BlockDim, BlockK, DataT, VW, MaxVW>>;
 
-        using MatrixLayout
-            = conditional_t<BlockDim <= 32u, SmallDimMatrixLayout, LargeDimMatrixLayout>;
+        // Select base layout from BlockDim
+        using BaseMatrixLayout
+            = conditional_t<LargeBlockDim, LargeDimMatrixLayout, SmallDimMatrixLayout>;
+
+        // Select final matrix layout from cooperative mode
+        using MatrixLayout = conditional_t<
+            SchedulerTraits::is_cooperative,
+            MatrixLayout::MatrixCoopLayout<BaseMatrixLayout, SchedulerTraits::WaveCount>,
+            BaseMatrixLayout>;
 
         // Register layout direct to memory storage (load / store)
         using StorageLayout = RegisterLayout::Storage<MatrixLayout, DataLayout>;
 
         // Register layout required for mma. Expect non-interleaved SOA format.
         // Quirk: gfx11 requires input duplication.
-        using MmaLayout = RegisterLayout::MmaInput<BlockDim,
-                                                   DataT,
-                                                   false,
-                                                   (bool)ROCWMMA_ARCH_GFX11
-                                                       ? RegisterLayout::Format::WMMA_INPUT_GFX11
-                                                       : RegisterLayout::Format::SOA>;
-        // Fragments will keep storage layout
-        using FragmentLayout = StorageLayout;
-    };
-
-    template <uint32_t BlockDim,
-              uint32_t BlockK,
-              typename DataT,
-              typename DataLayoutT,
-              uint32_t WaveCount>
-    struct IOLayout<matrix_b, BlockDim, BlockK, DataT, DataLayoutT, WaveCount>
-    {
-        // Vector size properties
-        constexpr static uint32_t MaxVW = detail::
-            MaxVWSelector<matrix_b, BlockDim, BlockK, DataT, DataLayoutT, WaveCount>::Result;
-
-        constexpr static uint32_t VW
-            = is_same_v<DataLayoutT, col_major> || BlockDim > 32 ? MaxVW : 1u;
-
-        // DataLayout
-        using DataLayout = DataLayout::template Array1d<DataLayoutT>;
-
-        // Matrix Layouts
-        // Small dim mma friendly
-        using SmallDimMatrixLayout =
-                            MatrixLayout::RowOrthoVW<BlockDim, BlockK, DataT, VW, MaxVW>;
-
-        // Large dim not mma friendly
-        using LargeDimMatrixLayout
-            = conditional_t<is_same_v<DataLayoutT, row_major>,
-                            MatrixLayout::RowInlineVW<BlockDim, BlockK, DataT, VW, MaxVW>,
-                            MatrixLayout::RowOrthoVW<BlockDim, BlockK, DataT, VW, MaxVW>>;
-
-        using MatrixLayout
-            = conditional_t<BlockDim <= 32, SmallDimMatrixLayout, LargeDimMatrixLayout>;
-
-        // Register layout direct to memory storage (load / store)
-        using StorageLayout = RegisterLayout::Storage<MatrixLayout, DataLayout>;
-
-        // Register layout required for mma. Expect non-interleaved SOA format.
-        // Quirk: gfx11 requires input duplication.
-        using MmaLayout = RegisterLayout::MmaInput<BlockDim,
+        using MmaLayout = RegisterLayout::MmaInput<MmaDim,
                                                    DataT,
                                                    false,
                                                    (bool)ROCWMMA_ARCH_GFX11
@@ -240,28 +221,112 @@ namespace rocwmma
               uint32_t BlockK,
               typename DataT,
               typename DataLayoutT,
-              uint32_t WaveCount>
-    struct IOLayout<accumulator, BlockDim, BlockK, DataT, DataLayoutT, WaveCount>
+              typename Scheduler>
+    struct IOLayout<matrix_b, BlockDim, BlockK, DataT, DataLayoutT, Scheduler>
     {
-        // Vector size properties
-        constexpr static uint32_t MaxVW = detail::
-            MaxVWSelector<accumulator, BlockDim, BlockK, DataT, DataLayoutT, WaveCount>::Result;
+        using SchedulerTraits = scheduler_traits<Scheduler>;
 
+        // For non-interleaved layout, MmaDim is given by BlockDim
+        constexpr static uint32_t MmaDim = BlockDim;
+
+        // Large dim affects our layout choice
+        constexpr static bool LargeBlockDim = BlockDim > 32u;
+
+        // Determine optimal vector width properties
+        constexpr static uint32_t MaxVW = detail::MaxVWSelector<matrix_b,
+                                                                BlockDim,
+                                                                BlockK,
+                                                                DataT,
+                                                                DataLayoutT,
+                                                                SchedulerTraits::WaveCount>::Result;
+
+        // Note: RowOrthoVW in row_major is only valid for VW = 1
+        constexpr static uint32_t VW
+            = (is_same_v<DataLayoutT, col_major> || LargeBlockDim) ? MaxVW : 1u;
+
+        // DataLayout selection
+        using DataLayout = DataLayout::template Array1d<DataLayoutT>;
+
+        // MatrixLayout selection depends on some factors:
+        // - BlockDim
+        // - Cooperative wave mode
+
+        // Small dim: ensure our layout mma friendly
+        using SmallDimMatrixLayout = MatrixLayout::RowOrthoVW<BlockDim, BlockK, DataT, VW, MaxVW>;
+
+        // Large dim: we can use higher BW layouts, not mma friendly
+        using LargeDimMatrixLayout
+            = conditional_t<is_same_v<DataLayoutT, row_major>,
+                            MatrixLayout::RowInlineVW<BlockDim, BlockK, DataT, VW, MaxVW>,
+                            MatrixLayout::RowOrthoVW<BlockDim, BlockK, DataT, VW, MaxVW>>;
+
+        // Select base layout from BlockDim
+        using BaseMatrixLayout
+            = conditional_t<LargeBlockDim, LargeDimMatrixLayout, SmallDimMatrixLayout>;
+
+        // Select final matrix layout from cooperative mode
+        using MatrixLayout = conditional_t<
+            SchedulerTraits::is_cooperative,
+            MatrixLayout::MatrixCoopLayout<BaseMatrixLayout, SchedulerTraits::WaveCount>,
+            BaseMatrixLayout>;
+
+        // Register layout direct to memory storage (load / store)
+        using StorageLayout = RegisterLayout::Storage<MatrixLayout, DataLayout>;
+
+        // Register layout required for mma. Expect non-interleaved SOA format.
+        // Quirk: gfx11 requires input duplication.
+        using MmaLayout = RegisterLayout::MmaInput<MmaDim,
+                                                   DataT,
+                                                   false,
+                                                   (bool)ROCWMMA_ARCH_GFX11
+                                                       ? RegisterLayout::Format::WMMA_INPUT_GFX11
+                                                       : RegisterLayout::Format::SOA>;
+
+        // Fragments will keep storage register layout.
+        using FragmentLayout = StorageLayout;
+    };
+
+    template <uint32_t BlockDim,
+              uint32_t BlockK,
+              typename DataT,
+              typename DataLayoutT,
+              typename Scheduler>
+    struct IOLayout<accumulator, BlockDim, BlockK, DataT, DataLayoutT, Scheduler>
+    {
+        using SchedulerTraits = scheduler_traits<Scheduler>;
+
+        // For non-interleaved layout, MmaDim is given by BlockDim
+        constexpr static uint32_t MmaDim = BlockDim;
+
+        // Vector size properties
+        constexpr static uint32_t MaxVW = detail::MaxVWSelector<accumulator,
+                                                                BlockDim,
+                                                                BlockK,
+                                                                DataT,
+                                                                DataLayoutT,
+                                                                SchedulerTraits::WaveCount>::Result;
+
+        // Note: RowOrthoVW in row_major is only valid for VW = 1
         constexpr static uint32_t VW = is_same_v<DataLayoutT, col_major> ? MaxVW : 1u;
 
         // DataLayout
         using DataLayout = DataLayout::template Array1d<DataLayoutT>;
 
         // Always mma friendly
-        using MatrixLayout
-            = MatrixLayout::RowOrthoVW<BlockDim, BlockK, DataT, VW, MaxVW>;
+        using BaseMatrixLayout = MatrixLayout::RowOrthoVW<BlockDim, BlockK, DataT, VW, MaxVW>;
+
+        // Select final matrix layout from cooperative mode
+        using MatrixLayout = conditional_t<
+            SchedulerTraits::is_cooperative,
+            MatrixLayout::MatrixCoopLayout<BaseMatrixLayout, SchedulerTraits::WaveCount>,
+            BaseMatrixLayout>;
 
         // Register layout direct to memory storage (load / store)
         using StorageLayout = RegisterLayout::Storage<MatrixLayout, DataLayout>;
 
         // Register layout required for mma. Expect non-interleaved SOA format.
         // Quirk: gfx11 requires padded acc.
-        using MmaLayout = RegisterLayout::MmaAcc<BlockDim,
+        using MmaLayout = RegisterLayout::MmaAcc<MmaDim,
                                                  DataT,
                                                  false,
                                                  (bool)ROCWMMA_ARCH_GFX11
@@ -272,15 +337,18 @@ namespace rocwmma
         using FragmentLayout = StorageLayout;
     };
 
-    template <uint32_t BlockDim, uint32_t BlockK, typename DataT, uint32_t WaveCount>
-    struct IOLayout<accumulator, BlockDim, BlockK, DataT, void, WaveCount>
+    template <uint32_t BlockDim, uint32_t BlockK, typename DataT, typename Scheduler>
+    struct IOLayout<accumulator, BlockDim, BlockK, DataT, void, Scheduler>
     {
+        // For non-interleaved layout, MmaDim is given by BlockDim
+        constexpr static uint32_t MmaDim = BlockDim;
+
         // We don't know which storage is needed: no DataLayout
         using StorageLayout = void;
 
         // Register layout required for mma. Expect non-interleaved SOA format.
         // Quirk: gfx11 requires padded acc.
-        using MmaLayout = RegisterLayout::MmaAcc<BlockDim,
+        using MmaLayout = RegisterLayout::MmaAcc<MmaDim,
                                                  DataT,
                                                  false,
                                                  (bool)ROCWMMA_ARCH_GFX11
@@ -288,10 +356,8 @@ namespace rocwmma
                                                      : RegisterLayout::Format::SOA>;
 
         // Fragments will assume default mma register layout.
-        using FragmentLayout = RegisterLayout::MmaAcc<BlockDim,
-                                                 DataT,
-                                                 false,
-                                                 RegisterLayout::Format::SOA>;
+        using FragmentLayout
+            = RegisterLayout::MmaAcc<MmaDim, DataT, false, RegisterLayout::Format::SOA>;
     };
 
     namespace detail
@@ -304,28 +370,32 @@ namespace rocwmma
         private:
             // Smallest valid mma dim for mfma/wmma.
             // Test MmaDim must not exceed BlockDim for valid layout.
-            static constexpr uint32_t MinMmaDim = 16u;
+            static constexpr uint32_t MinMmaDim  = 16u;
             static constexpr uint32_t TestMmaDim = min(BlockDim, MmaDim);
 
             // For valid mma sizes, (BlockDim >= 16)
             // Find minimum 16 byte load with MmaDim of 32 or 16
-            static constexpr uint32_t MinLargeBytes = 16u;
+            static constexpr uint32_t MinLargeBytes  = 16u;
             static constexpr uint32_t DimPerThread   = BlockDim / TestMmaDim;
             static constexpr uint32_t BytesPerThread = DimPerThread * sizeof(DataT);
-            static constexpr uint32_t MmaDimResult = (BytesPerThread < MinLargeBytes ? MinMmaDim : TestMmaDim);
+            static constexpr uint32_t MmaDimResult
+                = (BytesPerThread < MinLargeBytes ? MinMmaDim : TestMmaDim);
 
             // For invalid mma sizes (BlockDim < 16), we can have smaller MmaDim to increase VW.
             // Try to balance DimPerThread and KPerThread by aiming to get half BlockDim bytes.
-            static constexpr bool SmallDim = TestMmaDim < MinMmaDim;
+            static constexpr bool     SmallDim      = TestMmaDim < MinMmaDim;
             static constexpr uint32_t MinSmallBytes = BlockDim / 2u * sizeof(DataT);
-            static constexpr uint32_t SmallDimResult = (BytesPerThread < MinSmallBytes) ?
-                MmaDimSelector<BlockDim, DataT, TestMmaDim / 2u>::Result : TestMmaDim;
+            static constexpr uint32_t SmallDimResult
+                = (BytesPerThread < MinSmallBytes)
+                      ? MmaDimSelector<BlockDim, DataT, TestMmaDim / 2u>::Result
+                      : TestMmaDim;
 
         public:
-            static constexpr uint32_t Result = SmallDim ? SmallDimResult : MmaDimResult;
+            // TODO: Remove the min with 16u when MmaDim32 is implemented for interleaved layouts
+            static constexpr uint32_t Result = SmallDim ? SmallDimResult : min(MmaDimResult, 16u);
         };
 
-        template<uint32_t BlockDim, typename DataT>
+        template <uint32_t BlockDim, typename DataT>
         struct MmaDimSelector<BlockDim, DataT, 0u>
         {
             static constexpr uint32_t Result = 1u;
@@ -342,23 +412,28 @@ namespace rocwmma
     * @tparam BlockK Block K-dimension
     * @tparam DataT data type
     * @tparam DataLayoutT in-memory layout as col_major or row_major
-    * @tparam WaveCount number of cooperative waves
+    * @tparam Scheduler wave-wise scheduler
     */
     template <typename MatrixT,
               uint32_t BlockDim,
               uint32_t BlockK,
               typename DataT,
               typename DataLayoutT,
-              uint32_t WaveCount>
+              typename Scheduler>
     struct IOLayoutInt;
 
     template <uint32_t BlockDim,
               uint32_t BlockK,
               typename DataT,
               typename DataLayoutT,
-              uint32_t WaveCount>
-    struct IOLayoutInt<matrix_a, BlockDim, BlockK, DataT, DataLayoutT, WaveCount>
+              typename Scheduler>
+    struct IOLayoutInt<matrix_a, BlockDim, BlockK, DataT, DataLayoutT, Scheduler>
     {
+        using SchedulerTraits = scheduler_traits<Scheduler>;
+
+        // SplitK factor for interleaved layouts can be applied to WaveCount
+        constexpr static uint32_t SplitK = SchedulerTraits::WaveCount;
+
         // Select an appropriate MmaDim
         constexpr static uint32_t MmaDim = detail::MmaDimSelector<BlockDim, DataT>::Result;
 
@@ -366,10 +441,16 @@ namespace rocwmma
         using DataLayout = DataLayout::template Array1d<DataLayoutT>;
 
         // Matrix Layouts
-        using MatrixLayout
+        using BaseMatrixLayout
             = conditional_t<is_same_v<DataLayoutT, col_major>,
-                            MatrixLayout::ColInlineInt<BlockDim, BlockK, DataT, MmaDim, WaveCount>,
-                            MatrixLayout::ColOrthoInt<BlockDim, BlockK, DataT, MmaDim, WaveCount>>;
+                            MatrixLayout::ColInlineInt<BlockDim, BlockK, DataT, MmaDim, SplitK>,
+                            MatrixLayout::ColOrthoInt<BlockDim, BlockK, DataT, MmaDim, SplitK>>;
+
+        // Select final matrix layout from cooperative mode
+        using MatrixLayout = conditional_t<
+            SchedulerTraits::is_cooperative,
+            MatrixLayout::MatrixCoopLayout<BaseMatrixLayout, SchedulerTraits::WaveCount>,
+            BaseMatrixLayout>;
 
         // Register layout direct to memory storage (load / store)
         using StorageLayout = RegisterLayout::Storage<MatrixLayout, DataLayout>;
@@ -388,16 +469,21 @@ namespace rocwmma
 
         // Vector size properties derived from the matrix layout
         constexpr static uint32_t MaxVW = layout_traits<MatrixLayout>::MaxVectorWidth;
-        constexpr static uint32_t VW = MaxVW;
+        constexpr static uint32_t VW    = MaxVW;
     };
 
     template <uint32_t BlockDim,
               uint32_t BlockK,
               typename DataT,
               typename DataLayoutT,
-              uint32_t WaveCount>
-    struct IOLayoutInt<matrix_b, BlockDim, BlockK, DataT, DataLayoutT, WaveCount>
+              typename Scheduler>
+    struct IOLayoutInt<matrix_b, BlockDim, BlockK, DataT, DataLayoutT, Scheduler>
     {
+        using SchedulerTraits = scheduler_traits<Scheduler>;
+
+        // SplitK factor for interleaved layouts can be applied to WaveCount
+        constexpr static uint32_t SplitK = SchedulerTraits::WaveCount;
+
         // Select an appropriate MmaDim
         constexpr static uint32_t MmaDim = detail::MmaDimSelector<BlockDim, DataT>::Result;
 
@@ -405,10 +491,16 @@ namespace rocwmma
         using DataLayout = DataLayout::template Array1d<DataLayoutT>;
 
         // Matrix Layouts
-        using MatrixLayout
+        using BaseMatrixLayout
             = conditional_t<is_same_v<DataLayoutT, col_major>,
-                            MatrixLayout::RowOrthoInt<BlockDim, BlockK, DataT, MmaDim, WaveCount>,
-                            MatrixLayout::RowInlineInt<BlockDim, BlockK, DataT, MmaDim, WaveCount>>;
+                            MatrixLayout::RowOrthoInt<BlockDim, BlockK, DataT, MmaDim, SplitK>,
+                            MatrixLayout::RowInlineInt<BlockDim, BlockK, DataT, MmaDim, SplitK>>;
+
+        // Select final matrix layout from cooperative mode
+        using MatrixLayout = conditional_t<
+            SchedulerTraits::is_cooperative,
+            MatrixLayout::MatrixCoopLayout<BaseMatrixLayout, SchedulerTraits::WaveCount>,
+            BaseMatrixLayout>;
 
         // Register layout direct to memory storage (load / store)
         using StorageLayout = RegisterLayout::Storage<MatrixLayout, DataLayout>;
@@ -433,9 +525,14 @@ namespace rocwmma
               uint32_t BlockK,
               typename DataT,
               typename DataLayoutT,
-              uint32_t WaveCount>
-    struct IOLayoutInt<accumulator, BlockDim, BlockK, DataT, DataLayoutT, WaveCount>
+              typename Scheduler>
+    struct IOLayoutInt<accumulator, BlockDim, BlockK, DataT, DataLayoutT, Scheduler>
     {
+        using SchedulerTraits = scheduler_traits<Scheduler>;
+
+        // SplitK factor for interleaved layouts can be applied to WaveCount
+        constexpr static uint32_t SplitK = SchedulerTraits::WaveCount;
+
         // Select an appropriate MmaDim
         constexpr static uint32_t MmaDim = detail::MmaDimSelector<BlockDim, DataT>::Result;
 
@@ -443,38 +540,40 @@ namespace rocwmma
         using DataLayout = DataLayout::template Array1d<DataLayoutT>;
 
         // Matrix Layouts
-        using MatrixLayout
+        using BaseMatrixLayout
             = conditional_t<is_same_v<DataLayoutT, col_major>,
-                            MatrixLayout::RowOrthoInt<BlockDim, BlockK, DataT, MmaDim, WaveCount>,
-                            MatrixLayout::RowInlineInt<BlockDim, BlockK, DataT, MmaDim, WaveCount>>;
+                            MatrixLayout::RowOrthoInt<BlockDim, BlockK, DataT, MmaDim, SplitK>,
+                            MatrixLayout::RowInlineInt<BlockDim, BlockK, DataT, MmaDim, SplitK>>;
+
+        // Select final matrix layout from cooperative mode
+        using MatrixLayout = conditional_t<
+            SchedulerTraits::is_cooperative,
+            MatrixLayout::MatrixCoopLayout<BaseMatrixLayout, SchedulerTraits::WaveCount>,
+            BaseMatrixLayout>;
 
         // Register layout direct to memory storage (load / store)
         using StorageLayout = RegisterLayout::Storage<MatrixLayout, DataLayout>;
 
         // Register layout required for mma. Expect interleaved accum format for multiple blocks.
         // Quirk: gfx11 requires padded mma acc
-        using MmaLayout
-            = RegisterLayout::MmaAcc<MmaDim,
-                                     DataT,
-                                     true,
-                                     (bool)ROCWMMA_ARCH_GFX11
-                                         ? RegisterLayout::Format::WMMA_ACC_GFX11
-                                         : RegisterLayout::Format::ACC_INT_A_MAJOR>;
+        using MmaLayout = RegisterLayout::MmaAcc<MmaDim,
+                                                 DataT,
+                                                 true,
+                                                 (bool)ROCWMMA_ARCH_GFX11
+                                                     ? RegisterLayout::Format::WMMA_ACC_GFX11
+                                                     : RegisterLayout::Format::ACC_INT_A_MAJOR>;
 
         // Fragments will keep mma register layout.
         using FragmentLayout
-            = RegisterLayout::MmaAcc<MmaDim,
-                                     DataT,
-                                     true,
-                                     RegisterLayout::Format::ACC_INT_A_MAJOR>;
+            = RegisterLayout::MmaAcc<MmaDim, DataT, true, RegisterLayout::Format::ACC_INT_A_MAJOR>;
 
         // Vector size properties derived from the matrix layout
         constexpr static uint32_t MaxVW = layout_traits<MatrixLayout>::MaxVectorWidth;
         constexpr static uint32_t VW    = MaxVW;
     };
 
-    template <uint32_t BlockDim, uint32_t BlockK, typename DataT, uint32_t WaveCount>
-    struct IOLayoutInt<accumulator, BlockDim, BlockK, DataT, void, WaveCount>
+    template <uint32_t BlockDim, uint32_t BlockK, typename DataT, typename Scheduler>
+    struct IOLayoutInt<accumulator, BlockDim, BlockK, DataT, void, Scheduler>
     {
         // Select an appropriate MmaDim
         constexpr static uint32_t MmaDim = detail::MmaDimSelector<BlockDim, DataT>::Result;
@@ -484,19 +583,16 @@ namespace rocwmma
 
         // Register layout required for mma. Expect interleaved accum format for multiple blocks.
         // Quirk: gfx11 requires padded mma acc
-        using MmaLayout
-            = RegisterLayout::MmaAcc<MmaDim,
-                                     DataT,
-                                     true,
-                                     (bool)ROCWMMA_ARCH_GFX11
-                                         ? RegisterLayout::Format::WMMA_ACC_GFX11
-                                         : RegisterLayout::Format::ACC_INT_A_MAJOR>;
+        using MmaLayout = RegisterLayout::MmaAcc<MmaDim,
+                                                 DataT,
+                                                 true,
+                                                 (bool)ROCWMMA_ARCH_GFX11
+                                                     ? RegisterLayout::Format::WMMA_ACC_GFX11
+                                                     : RegisterLayout::Format::ACC_INT_A_MAJOR>;
 
         // Fragments will keep mma interleaved layout.
-        using FragmentLayout = RegisterLayout::MmaAcc<MmaDim,
-                                     DataT,
-                                     true,
-                                     RegisterLayout::Format::ACC_INT_A_MAJOR>;
+        using FragmentLayout
+            = RegisterLayout::MmaAcc<MmaDim, DataT, true, RegisterLayout::Format::ACC_INT_A_MAJOR>;
     };
 
 } // namespace rocwmma
