@@ -29,6 +29,8 @@
 
 #include "blend.hpp"
 #include "types.hpp"
+#include "utility/algorithm.hpp"
+#include "utility/type_traits.hpp"
 #include "utility/vector.hpp"
 
 namespace rocwmma
@@ -416,46 +418,204 @@ namespace rocwmma
         }
     }
 
-    // A permutation of vector indices, given a gather size and a stride
-    // Examples:
-    //                                     row_major               col_major
-    //       [0, 1] => interleave<1, 2>([0, 1, 2, 3, 4, 5])  =  [0, 2, 4, 1, 3, 5]
-    //   A = [2, 3]                        col_major               row_major
-    //       [4, 5] => interleave<1, 4>([0, 2, 4, 1, 3, 5])  =  [0, 1, 2, 3, 4, 5]
-    //
-    //       [0, 1]
-    //   A = [2, 3] => interleave<2, 4>([0, 1, 2, 3, 4, 5, 6, 7]) = [0, 1, 4, 5, 2, 3, 6, 7]
-    //       [4, 5]
-    //       [6, 7]
-    //
-    template <uint32_t GatherSize, uint32_t ElementStride, typename DataT, uint32_t VecSize>
-    ROCWMMA_DEVICE constexpr static inline decltype(auto) interleave(VecT<DataT, VecSize> const& v0)
+    // Interleaving index transform
+    template <uint32_t GatherSize, uint32_t ElementStride, uint32_t ElementCount>
+    struct interleave_idx
     {
-        static_assert((GatherSize >= 1u) && (GatherSize <= ElementStride)
-                          && (ElementStride % GatherSize == 0) && (VecSize % GatherSize == 0),
-                      "Invalid GatherSize");
-        static_assert(ElementStride >= 1u && ElementStride <= VecSize, "Invalid Stride");
-
-        // No transform is needed (NOP)
-        if constexpr(GatherSize == ElementStride || ElementStride == VecSize)
+        // Uses Number<I> abstraction for indices
+        template <typename NumberT>
+        constexpr static inline auto exec(NumberT&& Idx)
         {
-            return v0;
+            // Calculates offsets in interleave transform.
+            // If Index > ElementCount, offset cycle will repeat in-place.
+            // E.g., interleave<1, 2, 4> with input vector of size 8.
+            // ElementCount = 4, therefore offset cycle will repeat every 4 indices.
+            // Idx0 = 0  Idx4 = 4  Offset = 0
+            // Idx1 = 2  Idx5 = 6  Offset = 2
+            // Idx2 = 1  Idx6 = 5  Offset = 1
+            // Idx3 = 3  Idx7 = 7  Offset = 3
+            constexpr auto Index   = decay_t<decltype(Idx)>::value % ElementCount;
+            constexpr auto Offset0 = (Index / GatherSize) * ElementStride % ElementCount;
+            constexpr auto Offset1 = Index % GatherSize;
+            constexpr auto Offset2
+                = (Index * ElementStride) / (ElementCount * GatherSize) * GatherSize;
+            constexpr auto Offset3 = decay_t<decltype(Idx)>::value / ElementCount * ElementCount;
+            return I<Offset0 + Offset1 + Offset2 + Offset3>{};
+        }
+    };
+
+    template <typename IdXform>
+    struct interleave_idx_traits;
+
+    template <uint32_t GatherSizeIn, uint32_t ElementStrideIn, uint32_t ElementCountIn>
+    struct interleave_idx_traits<interleave_idx<GatherSizeIn, ElementStrideIn, ElementCountIn>>
+    {
+        static constexpr uint32_t GatherSize    = GatherSizeIn;
+        static constexpr uint32_t ElementStride = ElementStrideIn;
+        static constexpr uint32_t ElementCount  = ElementCountIn;
+
+        // NOP means transform is pass-through (no change)
+        static constexpr bool IsNop
+            = (GatherSize == ElementStride) || (ElementStride == ElementCount);
+
+        // Sanity check for params
+        static constexpr bool IsValid
+            = (GatherSize > 0u) && (ElementStride > 0u) && (ElementCount > 0u)
+              && (GatherSize <= ElementStride) && (GatherSize <= ElementCount)
+              && (ElementStride <= ElementCount) && (ElementStride % GatherSize == 0u)
+              && (ElementCount % GatherSize == 0u);
+    };
+
+    template <typename... IdXForms>
+    constexpr static inline bool test_interleave_idx_nop()
+    {
+        // Default value for && in fold is 'true', so this
+        // also covers the <> case, where nop is assumed.
+        return (interleave_idx_traits<IdXForms>::IsNop && ...);
+    }
+
+    template <typename... IdXForms>
+    constexpr static inline bool test_interleave_idx_valid()
+    {
+        // Default value for && in fold is 'true', which
+        // is not what we want in the <> case - it should be false
+        if constexpr(sizeof...(IdXForms) == 0u)
+        {
+            return false;
         }
         else
         {
-            auto offset = [](auto&& idx, auto&& v0) {
-                constexpr auto Index   = decay_t<decltype(idx)>::value;
-                constexpr auto Offset0 = (Index / GatherSize) * ElementStride % VecSize;
-                constexpr auto Offset1 = Index % GatherSize;
-                constexpr auto Offset2
-                    = (Index * ElementStride) / (VecSize * GatherSize) * GatherSize;
-                return get<Offset0 + Offset1 + Offset2>(v0);
+            return (interleave_idx_traits<IdXForms>::IsValid && ...);
+        }
+    }
+
+    // Given a set of interleave id xforms, combine them in the order given
+    template <typename... IdXForms>
+    struct idx_generator_fwd;
+
+    template <typename IdXForm, typename... IdXForms>
+    struct idx_generator_fwd<IdXForm, IdXForms...>
+    {
+        template <typename IdT>
+        constexpr static inline auto exec(IdT&& Idx)
+        {
+            return IdXForm::exec(idx_generator_fwd<IdXForms...>::exec(forward<IdT>(Idx)));
+        }
+    };
+
+    template <typename IdXForm>
+    struct idx_generator_fwd<IdXForm>
+    {
+        template <typename IdT>
+        constexpr static inline auto exec(IdT&& Idx)
+        {
+            return IdXForm::exec(forward<IdT>(Idx));
+        }
+    };
+
+    // Given a set of interleave id xforms, combine them in the reverse of the order given
+    template <typename... IdXForms>
+    struct idx_generator_bwd;
+
+    template <typename IdXForm, typename... IdXForms>
+    struct idx_generator_bwd<IdXForm, IdXForms...>
+    {
+        template <typename IdT>
+        constexpr static inline auto exec(IdT&& Idx)
+        {
+            return idx_generator_bwd<IdXForms...>::exec(IdXForm::exec(forward<IdT>(Idx)));
+        }
+    };
+
+    template <typename IdXForm>
+    struct idx_generator_bwd<IdXForm>
+    {
+        template <typename IdT>
+        constexpr static inline auto exec(IdT&& Idx)
+        {
+            return IdXForm::exec(forward<IdT>(Idx));
+        }
+    };
+
+    template <template <typename...> class IdxGenerator, typename... IdXForms, typename VecT>
+    ROCWMMA_DEVICE constexpr static inline decltype(auto) interleave_internal(VecT&& v0)
+    {
+        // Sanity check
+        static_assert(test_interleave_idx_valid<IdXForms...>(),
+                      "Invalid interleave xform provided");
+
+        if constexpr(test_interleave_idx_nop<IdXForms...>())
+        {
+            // If nop, don't make a copy
+            return forward<VecT>(v0);
+        }
+        else
+        {
+            // Interleave Index generator from from input transforms
+            using GenIdx = IdxGenerator<IdXForms...>;
+
+            // Extract vector traits
+            using VecTraits            = VecTraits<decay_t<decltype(v0)>>;
+            using DataT                = typename VecTraits::DataT;
+            constexpr uint32_t VecSize = VecTraits::size();
+
+            // Apply index generator
+            auto idxform = [](auto&& idx, auto&& v0) {
+                using Idx = decay_t<decltype(GenIdx::exec(idx))>;
+                return get<Idx>(forward<VecT>(v0));
             };
 
-            return vector_generator<DataT, VecSize>()(offset, v0);
+            // Generate a new vector with the xformed indices
+            return vector_generator<DataT, VecSize>()(idxform, forward<VecT>(v0));
         }
     }
 
+    template <typename... IdXForms, typename VecT>
+    ROCWMMA_DEVICE constexpr static inline decltype(auto) interleave_internal_fwd(VecT&& v0)
+    {
+        return interleave_internal<idx_generator_fwd, IdXForms...>(forward<VecT>(v0));
+    }
+
+    template <typename... IdXForms, typename VecT>
+    ROCWMMA_DEVICE constexpr static inline decltype(auto) interleave_internal_bwd(VecT&& v0)
+    {
+        return interleave_internal<idx_generator_bwd, IdXForms...>(forward<VecT>(v0));
+    }
+
+    // A permutation of vector indices, given a gather size and a stride
+    // Examples:
+    //                                     row_major               col_major
+    //       [0, 1] => interleave<1, 2, 6>([0, 1, 2, 3, 4, 5])  =  [0, 2, 4, 1, 3, 5]
+    //   A = [2, 3]                        col_major               row_major
+    //       [4, 5] => interleave<1, 4, 6>([0, 2, 4, 1, 3, 5])  =  [0, 1, 2, 3, 4, 5]
+    //
+    //       [0, 1]
+    //   A = [2, 3] => interleave<2, 4, 6>([0, 1, 2, 3, 4, 5, 6, 7]) = [0, 1, 4, 5, 2, 3, 6, 7]
+    //       [4, 5]
+    //       [6, 7]
+    //
+    template <uint32_t GatherSize,
+              uint32_t ElementStride,
+              uint32_t ElementCount = 0u,
+              typename VecT>
+    ROCWMMA_DEVICE constexpr static inline decltype(auto) interleave(VecT&& v0)
+    {
+        // If unspecified, ElementCount can be derived from the input vector size
+        using VecTraits            = VecTraits<decay_t<decltype(v0)>>;
+        constexpr uint32_t VecSize = ElementCount > 0u ? ElementCount : VecTraits::size();
+
+        // Build interleaved index generator
+        using IdXForm = interleave_idx<GatherSize, ElementStride, VecSize>;
+        return interleave_internal_fwd<IdXForm>(forward<VecT>(v0));
+    }
+
+    template <typename... IdXForms, typename VecT>
+    ROCWMMA_DEVICE constexpr static inline decltype(auto) interleave_combine(VecT&& v0)
+    {
+        // Build interleaved index generator
+        return interleave_internal_fwd<IdXForms...>(forward<VecT>(v0));
+    }
+
 } // namespace rocwmma
 
 #endif // ROCWMMA_VECTOR_UTIL_IMPL_HPP
